using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;

using CMS.SiteProvider;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.DataEngine;
using CMS.TreeEngine;
using CMS.Controls;
using CMS.UIControls;

public partial class CMSDesk_Content_Properties_MultipleCategoriesSelector : CMSUserControl
{
    #region "Variables"

    private bool mCombineWithGlobalCategories = true;
    private bool mSelectOnlyEnabled = true;
    private bool mHideOKButton = false;
    private bool mHideNewButton = false;
    private bool mAdministrationMode = false;
    private bool mFormControlMode = false;
    private int mPagerSize = 10;
    private int mDocumentID = 0;
    private int mUserID = 0;

    private DataSet dsUserCategories = null;
    private DataSet dsGlobalCategories = null;
    private DataSet dsDocumentCategories = null;

    private bool isSaved = false;

    #endregion


    #region "Properties"

    /// <summary>
    /// ID of the category to edit
    /// </summary>
    public int CategoryID
    {
        get
        {
        	 return ValidationHelper.GetInteger(ViewState["CategoryID"], 0); 
        }
        set
        {
        	 ViewState["CategoryID"] = value; 
        }
    }


    /// <summary>
    /// Gets or sets the DocumentID for which the categories should be loaded
    /// </summary>
    public int DocumentID
    {
        get
        {
            return this.mDocumentID;
        }
        set
        {
            this.mDocumentID = value;
        }
    }


    /// <summary>
    /// Gets or sets the UserID whose categories should be displayed
    /// </summary>
    public int UserID
    {
        get
        {
            if (this.mUserID > 0)
            {
                return this.mUserID;
            }
            else
            {
                return CMSContext.CurrentUser.UserID;
            }
        }
        set
        {
            this.mUserID = value;
        }
    }


    /// <summary>
    /// Gets or sets the number of categories displayed per page
    /// </summary>
    public int PagerSize
    {
        get
        {
            return this.mPagerSize;
        }
        set
        {
            this.mPagerSize = value;
        }
    }


    /// <summary>
    /// Gets or sets the value which determines whether to show also global categories
    /// </summary>
    public bool CombineWithGlobalCategories
    {
        get
        {
            return this.mCombineWithGlobalCategories;
        }
        set
        {
            this.mCombineWithGlobalCategories = value;
        }
    }


    /// <summary>
    /// Gets or sets the value which determines whether to display only enabled categories
    /// </summary>
    public bool SelectOnlyEnabled
    {
        get
        {
            return this.mSelectOnlyEnabled;
        }
        set
        {
            this.mSelectOnlyEnabled = value;
        }
    }


    /// <summary>
    /// Gets or sets the value which determines whether to display OK button or not
    /// </summary>
    public bool HideOKButton
    {
        get
        {
            return this.mHideOKButton;
        }
        set
        {
            this.mHideOKButton = value;
        }
    }


    /// <summary>
    /// Gets or sets the value which determines whether to display New button or not
    /// </summary>
    public bool HideNewButton
    {
        get
        {
            return this.mHideNewButton;
        }
        set
        {
            this.mHideNewButton = value;
        }
    }


    /// <summary>
    /// Gets or sets the value which determines if administration mode is On or not
    /// </summary>
    public bool AdministrationMode
    {
        get
        {
            return this.mAdministrationMode;
        }
        set
        {
            this.mAdministrationMode = value;
        }
    }


    /// <summary>
    /// Gets or sets the value which determines if form control mode is On or not
    /// </summary>
    public bool FormControlMode
    {
        get
        {
            return this.mFormControlMode;
        }
        set
        {
            this.mFormControlMode = value;
        }
    }


    /// <summary>
    /// Gets or sets the text whixh is displayed when no actegories found
    /// </summary>
    public string ZeroRowsText
    {
        get
        {
            return this.gridUserCategory.EmptyDataText;
        }
        set
        {
            this.gridUserCategory.EmptyDataText = value;
            this.gridGlobalCategory.EmptyDataText = value;
        }
    }


    /// <summary>
    /// Gets or sets User categories unigrid.
    /// </summary>
    public Control GridMyCategories
    {
        get
        {
            return ((Control)this.gridUserCategory);
        }
        set
        {
            this.gridUserCategory = (GridView)value;
        }
    }


    /// <summary>
    /// Gets or sets Global categories unigrid.
    /// </summary>
    public Control GridGlobalCategories
    {
        get
        {
            return ((Control)this.gridGlobalCategory);
        }
        set
        {
            this.gridGlobalCategory = (GridView)value;
        }
    }


    /// <summary>
    /// Return true if editing form is visible.
    /// </summary>
    public bool Editing
    {
        get
        {
            return this.pnlEdit.Visible;
        }
    }

    #endregion


    #region "Events"
    
    /// <summary>
    /// Occurs after data is saved to the database.
    /// </summary>
    public delegate void OnAfterSaveEventHandler();

    /// <summary>
    /// OnAfterSave event.
    /// </summary>
    public event OnAfterSaveEventHandler OnAfterSave;

    #endregion


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        if (RequestHelper.IsPostBack() && !isSaved)
        {
            this.dsDocumentCategories = CategoryInfoProvider.GetDocumentCategories(this.mDocumentID, null, null);
            if (!this.FormControlMode)
            {
                LoadCategories();
            }

            // Set category id from edit element
            if (this.categoryEditElem.CategoryID > 0)
            {
                this.CategoryID = this.categoryEditElem.CategoryID;
            }

            if (this.CategoryID > 0)
            {
                CategoryInfo categoryInfo = CategoryInfoProvider.GetCategoryInfo(this.CategoryID);
                if (categoryInfo != null)
                {
                    this.lblCurrent.Text = HTMLHelper.HTMLEncode(categoryInfo.CategoryDisplayName);
                }
            }
            else
            {
                this.lblCurrent.Text = ResHelper.GetString("MultipleCategoriesSelector.NewCategory");
            }
        }

        if ((!this.FormControlMode)&&(isSaved))
        {
            // Changes saved
            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
            pnlInfo.Visible = true;
        }

        // If no "My categories" presents
        if (this.gridUserCategory.Rows.Count == 0)
        {
            // Hide "My categories" title
            this.pnlUserGrid.Visible = false;

            // If user is not global admin
            if (!CMSContext.CurrentUser.IsGlobalAdministrator)
            {
                // Hide actions column in global gridview
                this.gridGlobalCategory.Columns[1].Visible = false;
            }
        }
        else
        {
            if ((!this.gridGlobalCategory.Columns[1].Visible)&&(!this.FormControlMode))
            {
                // Show actions column in global gridview if hidden
                this.gridGlobalCategory.Columns[1].Visible = true;
            }
        }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
        if (!this.StopProcessing)
        {
            // Set category id from edit element
            if (this.categoryEditElem.CategoryID > 0)
            {
                this.CategoryID = this.categoryEditElem.CategoryID;
            }

            isSaved = QueryHelper.GetBoolean("saved", false);

            this.lblUserTitle.Text = ResHelper.GetString("Categories.UserCategories");
            this.lnkList.Text = ResHelper.GetString("Categories.CategoriesList");
            this.lblGlobalTitle.Text = ResHelper.GetString("Categories.GlobalCategories");
            this.imgNewItem.ImageUrl = "~/App_Themes/" + this.Page.Theme + "/Images/CMSSiteManager/Development/Categories/addcategory.gif";
            this.imgNewItem.AlternateText = ResHelper.GetString("MultipleCategoriesSelector.NewCategory");

            this.gridUserCategory.RowDataBound += new GridViewRowEventHandler(gridUserCategory_RowDataBound);
            this.gridUserCategory.RowEditing += new GridViewEditEventHandler(gridUserCategory_RowEditing);
            this.gridGlobalCategory.RowDataBound += new GridViewRowEventHandler(gridGlobalCategory_RowDataBound);
            this.gridGlobalCategory.RowEditing += new GridViewEditEventHandler(gridGlobalCategory_RowEditing);

            this.categoryEditElem.UserID = this.UserID;
            this.categoryEditElem.CategoryID = this.CategoryID;
            this.categoryEditElem.ShowCodeName = false;
            this.categoryEditElem.ShowEnabled = false;
            this.categoryEditElem.OnSaved += new EventHandler(categoryEditElem_OnSaved);

            this.gridUserCategory.PageSize = this.PagerSize;
            this.gridGlobalCategory.PageSize = this.PagerSize;

            // Delete confirmation
            ltlScript.Text = ScriptHelper.GetScript("deleteConfirmation = '" + ResHelper.GetString("Categories.DeleteConfirmation") + "';");

            this.Page.ClientScript.RegisterClientScriptBlock(typeof(string), "DeleteScript", ScriptHelper.GetScript("function DeleteConfirm() { return confirm(deleteConfirmation); }"));

            // Get categories for the document type
            this.dsDocumentCategories = CategoryInfoProvider.GetDocumentCategories(this.mDocumentID, null, null);

            if (this.AdministrationMode)
            {
                // Hide global unigrid and label
                gridGlobalCategory.Visible = false;
                lblGlobalTitle.Visible = false;

                // Hide selector
                gridUserCategory.Columns[2].Visible = false;

                // Hide my categories title
                this.lblUserTitle.Visible = false;

                // Hide Ok button
                this.HideOKButton = true;

                // Show checkbox enabled in edit
                this.categoryEditElem.ShowEnabled = true;

                // Show all categories
                this.SelectOnlyEnabled = false;
            }
            else
            {
                if (this.FormControlMode)
                {
                    this.HideNewButton = true;
                    this.HideOKButton = true;
                    this.gridGlobalCategory.Columns[1].Visible = false;
                    this.gridUserCategory.Columns[1].Visible = false;
                }
            }

            // Hide OK button if specified
            if (!this.HideOKButton)
            {
                // Disable OK button if no categories found
                this.btnOK.Text = ResHelper.GetString("General.OK");
                if ((DataHelper.DataSourceIsEmpty(dsUserCategories)) && (DataHelper.DataSourceIsEmpty(dsGlobalCategories)))
                {
                    this.btnOK.Enabled = false;
                }
            }
            else
            {
                this.btnOK.Visible = false;
            }

            // Hide new button if specified
            if (!this.HideNewButton)
            {
                this.lnkNewItem.Text = ResHelper.GetString("MultipleCategoriesSelector.NewCategory");
            }
            else
            {
                this.pnlNewItem.Visible = false;
            }

            // Show edit category control if needed
            if (this.CategoryID > 0)
            {
                this.categoryEditElem.CategoryID = this.CategoryID;
                this.categoryEditElem.UserID = CMSContext.CurrentUser.UserID;

                CategoryInfo categoryInfo = CategoryInfoProvider.GetCategoryInfo(this.CategoryID);
                if (categoryInfo != null)
                {
                    this.lblCurrent.Text = HTMLHelper.HTMLEncode(categoryInfo.CategoryName);
                }
            }
            else
            {
                this.lblCurrent.Text = ResHelper.GetString("MultipleCategoriesSelector.NewCategory");
            }

            // Load categories
            LoadCategories();
        }
    }


    /// <summary>
    /// Edit button click
    /// </summary>
    protected void btnEdit_Command(object sender, CommandEventArgs e)
    {
        int categoryId = ValidationHelper.GetInteger(e.CommandArgument, 0);

        this.CategoryID = categoryId;
        this.categoryEditElem.CategoryID = categoryId;
        this.categoryEditElem.ReloadData();
        this.pnlEdit.Visible = true;
        this.pnlUserGrid.Visible = false;
        this.pnlGlobalGrid.Visible = false;
        this.pnlNewItem.Visible = false;
        this.pnlBreadCrumb.Visible = true;

        this.pnlInfo.Visible = false;
        isSaved = false;
    }


    /// <summary>
    /// Edit button click
    /// </summary>
    protected void btnDelete_Command(object sender, CommandEventArgs e)
    {
        // Get category Id
        int categoryId = ValidationHelper.GetInteger(e.CommandArgument, 0);

        // Delete category from DB
        CategoryInfoProvider.DeleteCategoryInfo(categoryId);

        // We need to refresh the page to reload all categories
        if (!this.FormControlMode)
        {
            UrlHelper.Redirect(UrlHelper.AddParameterToUrl(UrlHelper.CurrentURL, "saved", "0"));
        }
    }


    /// <summary>
    /// OK button click - save the values
    /// </summary>
    protected void lnkNewItem_Click(object sender, EventArgs e)
    {
        this.CategoryID = 0;
        this.categoryEditElem.ShowEnabled = false;
        this.categoryEditElem.CategoryID = 0;
        this.categoryEditElem.UserID = CMSContext.CurrentUser.UserID;
        this.categoryEditElem.ReloadData();
        this.pnlEdit.Visible = true;
        this.pnlUserGrid.Visible = false;
        this.pnlGlobalGrid.Visible = false;
        this.pnlNewItem.Visible = false;
        this.pnlBreadCrumb.Visible = true;
        this.pnlInfo.Visible = false;
        isSaved = false;
    }


    void categoryEditElem_OnSaved(object sender, EventArgs e)
    {
        lnkList_Click(sender, e);
    }


    /// <summary>
    /// lnkList click event handler.
    /// </summary>
    protected void lnkList_Click(object sender, EventArgs e)
    {
        this.pnlEdit.Visible = false;
        this.pnlUserGrid.Visible = true;
        this.pnlGlobalGrid.Visible = true;
        this.pnlNewItem.Visible = true;
        this.pnlBreadCrumb.Visible = false;

        // We need to refresh the page to reload all categories
        if (!this.FormControlMode)
        {
            UrlHelper.Redirect(UrlHelper.AddParameterToUrl(UrlHelper.CurrentURL, "saved", "0"));
        }
    }


    /// <summary>
    /// OK button click - save the values
    /// </summary>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        Save();
    }


    /// <summary>
    /// Loads the categories
    /// </summary>
    public void LoadCategories()
    {
        LoadCategories(false, false);
    }


    /// <summary>
    /// Loads the categories
    /// </summary>
    /// <param name="forceLoadUser">Indicate if user categories should be force load.</param>
    /// <param name="forceLoadGlobal">Indicate if global categories should be force load.</param>
    public void LoadCategories(bool forceLoadUser,bool forceLoadGlobal)
    {
        // Create where condition
        string where = (this.mSelectOnlyEnabled ? "CategoryEnabled = 1" : null);

        if ((gridUserCategory.Rows.Count == 0) || forceLoadUser)
        {
            // Get all categories for given user
            this.dsUserCategories = CategoryInfoProvider.GetCategories(CreateWhereCondition(where, false), "CategoryDisplayName");
            gridUserCategory.DataSource = this.dsUserCategories;
            gridUserCategory.DataBind();
        }

        if ((gridGlobalCategory.Rows.Count == 0) || forceLoadGlobal)
        {
            // Get all global categories
            this.dsGlobalCategories = CategoryInfoProvider.GetCategories(CreateWhereCondition(where, true), "CategoryDisplayName");
            gridGlobalCategory.DataSource = this.dsGlobalCategories;
            gridGlobalCategory.DataBind();
        }
    }


    /// <summary>
    /// OK button click - save the values
    /// </summary>
    public void Save()
    {
        // Check "modify" permission
        if ((this.UserID != CMSContext.CurrentUser.UserID) && (!CMSContext.CurrentUser.IsAuthorizedPerResource("CMS.Users", "Modify")))
        {
            RedirectToAccessDenied("CMS.Users", "Modify");
        } 

        // Go through all categories and update the relationship with the document
        foreach (GridViewRow row in gridUserCategory.Rows)
        {
            // Get CategoryID from the third column
            int categoryId = ValidationHelper.GetInteger(((Label)row.Cells[3].FindControl("lblCategoryID")).Text, 0);
            try
            {
                CheckBox checkbox = row.Cells[2].FindControl("chkIsInCategory") as CheckBox;
                if (checkbox != null)
                {
                    bool docIsInCategory = DocumentIsInCategory(categoryId);
                    if (checkbox.Checked && !docIsInCategory)
                    {
                        // Add document to category
                        DocumentCategoryInfoProvider.AddDocumentToCategory(this.mDocumentID, categoryId);
                    }
                    else if (!checkbox.Checked && docIsInCategory)
                    {
                        // Remove document from category
                        DocumentCategoryInfoProvider.RemoveDocumentFromCategory(this.mDocumentID, categoryId);
                    }
                }
            }
            catch { }
        }

        // Go through all categories and update the relationship with the document
        foreach (GridViewRow row in gridGlobalCategory.Rows)
        {
            // Get CategoryID from the third column
            int categoryId = ValidationHelper.GetInteger(((Label)row.Cells[3].FindControl("lblCategoryID")).Text, 0);
            try
            {
                CheckBox checkbox = row.Cells[2].FindControl("chkIsInCategory") as CheckBox;
                if (checkbox != null)
                {
                    bool docIsInCategory = DocumentIsInCategory(categoryId);
                    if (checkbox.Checked && !docIsInCategory)
                    {
                        // Add document to category
                        DocumentCategoryInfoProvider.AddDocumentToCategory(this.mDocumentID, categoryId);
                    }
                    else if (!checkbox.Checked && docIsInCategory)
                    {
                        // Remove document from category
                        DocumentCategoryInfoProvider.RemoveDocumentFromCategory(this.mDocumentID, categoryId);
                    }
                }
            }
            catch { }
        }

        // Raise on after save
        if (OnAfterSave != null)
        {
            OnAfterSave();
        }

        if (!this.FormControlMode)
        {
            // Refresh page
            UrlHelper.Redirect(UrlHelper.AddParameterToUrl(UrlHelper.CurrentURL,"saved","1"));
        }

        isSaved = true;
    }


    /// <summary>
    /// Returns string of categories ID
    /// </summary>
    public string GetSelectedCategories()
    {
        StringBuilder builder = new StringBuilder();

        // Go through all user categories
        foreach (GridViewRow row in gridUserCategory.Rows)
        {
            // Get CategoryID from the third column
            int categoryId = ValidationHelper.GetInteger(((Label)row.Cells[3].FindControl("lblCategoryID")).Text, 0);
            builder.Append(categoryId + ",");
        }

        // Go through all global categories
        foreach (GridViewRow row in gridGlobalCategory.Rows)
        {
            // Get CategoryID from the third column
            int categoryId = ValidationHelper.GetInteger(((Label)row.Cells[3].FindControl("lblCategoryID")).Text, 0);
            builder.Append(categoryId + ",");
        }

        // Return categories
        return builder.ToString().TrimEnd(',');
    }


    #region "Private helper functions"

    /// <summary>
    /// Determines whether specified category is assigned to the current document
    /// </summary>
    /// <param name="categoryId">Option category ID.</param>
    private bool DocumentIsInCategory(int categoryId)
    {
        if (!DataHelper.DataSourceIsEmpty(dsDocumentCategories))
        {
            return (dsDocumentCategories.Tables[0].Select("CategoryID = " + categoryId).Length > 0);
        }
        return false;
    }


    /// <summary>
    /// Return where condition with userID and combine with global.
    /// </summary>
    /// <param name="where">Main where condition</param>
    /// <param name="global">Global categories</param>
    private string CreateWhereCondition(string where, bool global)
    {
        string whereText = "(";

        // if the user was specified
        if ((this.UserID > 0) && (!global))
        {
            whereText += "CategoryUserID = " + this.UserID;
        }
        else
        {
            whereText += "CategoryUserID IS NULL";
        }

        whereText += ")";

        // if some user was specified
        if (whereText.Trim() != "()")
        {
            if ((where != null) && (where.Trim() != ""))
            {
                whereText += " AND (" + where + ")";
            }
        }
        else
        {
            whereText = where;
        }

        return whereText;
    }

    #endregion


    #region "Grid events"

    /// <summary>
    /// RowDataboud event
    /// </summary>
    protected void gridUserCategory_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            DataRow dr = ((DataRowView)e.Row.DataItem).Row;

            // Get option category ID
            int categoryId = ValidationHelper.GetInteger(dr["CategoryID"], 0);

            // Initialize controls in grid
            CheckBox checkbox = e.Row.Cells[2].FindControl("chkIsInCategory") as CheckBox;
            if (checkbox != null)
            {
                checkbox.Checked = DocumentIsInCategory(categoryId);
            }
            ImageButton editButton = e.Row.Cells[2].FindControl("btnEdit") as ImageButton;
            if (editButton != null)
            {
                if (this.UserID == ValidationHelper.GetInteger(dr["CategoryUserId"], 0))
                {
                    editButton.Visible = true;
                }

                editButton.ImageUrl = "~/App_Themes/" + Page.Theme + "/Images/CMSAdminControls/UniGrid/edit.gif";
                editButton.ToolTip = ResHelper.GetString("MultipleCategoriesSelector.Edit");
                editButton.CommandArgument = categoryId.ToString();
            }
            ImageButton deleteButton = e.Row.Cells[2].FindControl("btnDelete") as ImageButton;
            if (deleteButton != null)
            {
                if (this.UserID == ValidationHelper.GetInteger(dr["CategoryUserId"], 0))
                {
                    deleteButton.Visible = true;
                }

                deleteButton.ImageUrl = "~/App_Themes/" + Page.Theme + "/Images/CMSAdminControls/UniGrid/delete.gif";
                deleteButton.ToolTip = ResHelper.GetString("MultipleCategoriesSelector.Delete");
                deleteButton.CommandArgument = categoryId.ToString();
            }
            Label label = e.Row.Cells[3].FindControl("lblCategoryID") as Label;
            if (label != null)
            {
                label.Text = ValidationHelper.GetString(dr["CategoryID"], "0");
            }
        }
    }


    /// <summary>
    /// RowDataboud event
    /// </summary>
    protected void gridGlobalCategory_RowDataBound(object sender, GridViewRowEventArgs e)
    {
        if (e.Row.RowType == DataControlRowType.DataRow)
        {
            DataRow dr = ((DataRowView)e.Row.DataItem).Row;

            // Get option category ID
            int categoryId = ValidationHelper.GetInteger(dr["CategoryID"], 0);

            // Initialize controls in grid
            CheckBox checkbox = e.Row.Cells[2].FindControl("chkIsInCategory") as CheckBox;
            if (checkbox != null)
            {
                checkbox.Checked = DocumentIsInCategory(categoryId);
            }
            ImageButton editButton = e.Row.Cells[2].FindControl("btnEdit") as ImageButton;
            if (editButton != null)
            {
                UserInfo currentUser = CMSContext.CurrentUser;
                if (currentUser.UserIsGlobalAdministrator)
                {
                    editButton.Visible = true;
                }

                editButton.ImageUrl = "~/App_Themes/" + Page.Theme + "/Images/CMSAdminControls/UniGrid/edit.gif";
                editButton.ToolTip = ResHelper.GetString("MultipleCategoriesSelector.Edit");
                editButton.CommandArgument = categoryId.ToString();
            }
            Label label = e.Row.Cells[3].FindControl("lblCategoryID") as Label;
            if (label != null)
            {
                label.Text = ValidationHelper.GetString(dr["CategoryID"], "0");
            }
        }
    }


    /// <summary>
    /// PageIndexChanging event
    /// </summary>
    protected void gridUserCategory_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        gridUserCategory.PageIndex = e.NewPageIndex;
        LoadCategories(true, false);
    }


    /// <summary>
    /// PageIndexChanging event
    /// </summary>
    protected void gridGlobalCategory_PageIndexChanging(object sender, GridViewPageEventArgs e)
    {
        gridGlobalCategory.PageIndex = e.NewPageIndex;
        LoadCategories(false, true);
    }


    /// <summary>
    /// RowUpdating event
    /// </summary>
    protected void gridUserCategory_RowEditing(object sender, GridViewEditEventArgs e)
    {
        // editing is handled in custom method (btnEdit_Command)
    }


    /// <summary>
    /// RowUpdating event
    /// </summary>
    protected void gridGlobalCategory_RowEditing(object sender, GridViewEditEventArgs e)
    {
        // editing is handled in custom method (btnEdit_Command)
    }

    #endregion
}
